@@ -21,10 +21,22 @@ from time import time
 import dill
 from pathos.multiprocessing import ProcessingPool as Pool
 
+def module_source_target(source, target):
+    if isinstance(source, str) and ':' in source:
+        source = tuple(source.split(':'))
+    if isinstance(target, str) and ':' in target:
+        target = tuple(target.split(':'))
+    return source, target
+
+def module_state(state):
+    if isinstance(state, str) and ':' in state:
+        state = tuple(state.split(':'))
+    return state
+
 def precache(transition_datas):
     for tran_datas in transition_datas:
-        tran_datas['natex'].precache()
-    parsed_trees = [x['natex']._compiler._parsed_tree for x in transition_datas]
+        tran_datas['question_natex'].precache()
+    parsed_trees = [x['question_natex']._compiler._parsed_tree for x in transition_datas]
     return parsed_trees
 
 _autostate = '-1'
@@ -48,21 +60,24 @@ class DialogueFlow:
         return _autostate
 
     def __init__(self, initial_state: Union[Enum, str, tuple], initial_speaker = Speaker.SYSTEM,
-                 macros: Dict[str, Macro] =None, kb: Union[KnowledgeBase, str, List[str]] =None):
+                 macros: Dict[str, Macro] =None, kb: Union[KnowledgeBase, str, List[str]] =None, default_system_state=None):
         self._graph = GraphDatabase()
         self._initial_state = State(initial_state)
-        self._state = self._initial_state
         self._potential_transition = None
         self._initial_speaker = initial_speaker
         self._speaker = self._initial_speaker
         self._response = None
-        self._vars = {}
+        self._vars = HashableDict()
+        self.set_state(self._initial_state)
         self._gate_requirements = defaultdict(dict)
         self._gates = defaultdict(set)
         self._gate_buffer = {}
+        self._prepends = {}
         self._var_dependencies = defaultdict(set)
         self._error_transitioned = False
-        self._prepends = {}
+        self._is_module = False
+        self._default_state = default_system_state
+        self.vars()['__stack__'] = []
         if kb is None:
             self._kb = KnowledgeBase()
         elif isinstance(kb, str):
@@ -101,11 +116,18 @@ class DialogueFlow:
             'POS': PartOfSpeech(),
             'LEM': Lemma(),
             'SCORE': Score(),
-            'TOKLIMIT': TokLimit()
+            'TOKLIMIT': TokLimit(),
+            'AGREE': Agree(),
+            'DISAGREE': Disagree(),
+            'QUESTION': Question(),
+            'NEGATION': Negation(),
+            'IDK': DontKnow(),
+            'MAYBE': Maybe(),
+            'TRANSITION': Transition(),
+            'UNX': Unexpected()
         }
         if macros:
             self._macros.update(macros)
-        self._global_states = set()
         self._rules = UpdateRules(vars=self._vars, macros=self._macros)
 
     # TOP LEVEL: SYSTEM-LEVEL USE CASES
@@ -175,6 +197,7 @@ class DialogueFlow:
                 self.change_speaker()
                 break
             visited.add(next_state)
+        self.set_speaker(Speaker.SYSTEM)
         t2 = time()
         if debugging:
             print('User turn in {:.5f}'.format(t2 - t1))
@@ -194,6 +217,7 @@ class DialogueFlow:
             root = self._initial_state
 
         hop = None
+        switch = False
 
         # read settings and transitions for state
         transitions = []
@@ -208,7 +232,9 @@ class DialogueFlow:
             elif key == 'prepend':
                 prepend = json_dict['prepend']
                 self.set_state_prepend(source, prepend)
-            elif key not in {'state', 'hop', 'score'}:
+            elif key == 'switch':
+                switch = json_dict['switch']
+            elif key not in {'state', 'hop', 'score', 'switch'}:
                 transitions.append((key, value))
 
         # set up state settings
@@ -222,10 +248,19 @@ class DialogueFlow:
             elif speaker == Speaker.SYSTEM:
                 speaker = Speaker.USER
                 self.state_settings(source).update(user_multi_hop=True)
+        if switch:
+            self.update_state_settings(source, switch=True)
 
         # set up transitions
         expanded_transitions = []
         for natex, target in transitions:
+            natex_with_leading_digits_stripped = ''
+            i = 0
+            c = natex[i] if natex else ''
+            while c and c.isnumeric():
+                natex_with_leading_digits_stripped += c
+                i += 1
+                c = natex[i] if i < len(natex) else ''
             if natex == 'error':
                 if isinstance(target, dict):
                     if 'state' not in target:
@@ -283,21 +318,27 @@ class DialogueFlow:
         :param debugging:
         :return: a <state, response> tuple representing the successor state and response
         """
+        state = module_state(state)
         ti = time()
         if state is None:
-            state = self._state
+            state = self.state()
         else:
             state = State(state)
         transition_options = {}
         if self._response is not None:
             response, vars, dest, score = self._response
-            transition_options[(response, (self._state, dest, self._speaker), vars)] = score
+            transition_options[(response, (self.state(), dest, self._speaker), vars)] = score
         self._gate_buffer.clear()
         transitions = list(self.transitions(state, Speaker.SYSTEM))
         for transition in transitions:
             self._potential_transition = transition
+            if not self._is_module and isinstance(transition[1], tuple):
+                continue
             t1 = time()
             natex = self.transition_natex(*transition)
+            if '->' in transition[1]:
+                transition = (transition[1].split('->')[0], transition[1].split('->')[1], Speaker.SYSTEM)
+                natex = natex + self.transition_natex(*transition)
             settings = self.transition_settings(*transition)
             vars = HashableDict(self._vars)
             try:
@@ -319,6 +360,7 @@ class DialogueFlow:
             if debugging:
                 print('Transition {} evaluated in {:.5f}'.format(transition, t2-t1))
         if transition_options:
+            remaining = list(transition_options)
             memory = self.state_settings(state).memory
             for item in memory:
                 if len(transition_options) > 1:
@@ -332,6 +374,17 @@ class DialogueFlow:
                 else:
                     break
             response, transition, vars = random_max(transition_options, key=lambda x: transition_options[x])
+            if self.is_switch(state):
+                to_add_to_stack = [x[1] for x in remaining if x != (response, transition, vars)]
+                for trans in to_add_to_stack:
+                    src, tgt, natex = trans[0], trans[1], self.transition_natex(*trans)
+                    if isinstance(src, tuple):
+                        src = ':'.join(src)
+                    if isinstance(tgt, tuple):
+                        tgt = ':'.join(tgt)
+                    if src != state:
+                        tgt = src + '->' + tgt
+                    self.vars()['__stack__'].append(tgt)
             if debugging:
                 updates = {}
                 for k, v in vars.items():
@@ -352,9 +405,56 @@ class DialogueFlow:
                 tf = time()
                 print('System transition in {:.5f}'.format(tf-ti))
                 print('Transitioning {} -> {}'.format(self.state(), next_state))
+            if self._response is not None:
+                self._response = None
+            if '__response_prefix__' in self.vars() and self.vars()['__response_prefix__'] != 'None':
+                response = self.vars()['__response_prefix__'] + ' ' + response
+                self.vars()['__response_prefix__'] = 'None'
+            if self.is_switch(state):
+                switch_response, switch_next_state = self.system_transition(next_state)
+                return response + ' ' + switch_response, switch_next_state
             return response, next_state
         else:
-            raise AssertionError('dialogue flow system transition found no valid options')
+            while self.is_switch(state) and self.vars()['__stack__']:
+                target = self.vars()['__stack__'].pop()
+                if '->' in target:
+                    source, target = target.split('->')
+                    transition = source, target, Speaker.SYSTEM
+                    natex = self.transition_natex(*transition)
+                    vars = HashableDict(self._vars)
+                    try:
+                        response = natex.generate(vars=vars, macros=self._macros, debugging=debugging)
+                    except Exception as e:
+                        print()
+                        print(e)
+                        print('Stack transition to {}: {} failed'.format(str(target), natex))
+                        print()
+                        response = None
+                    if response is not None:
+                        if transition in self.gate_buffer():
+                            self.gates()[transition].add(self.gate_buffer()[transition])
+                        self.update_vars(vars)
+                        next_state = transition[1]
+                        if debugging:
+                            tf = time()
+                            print('System transition in {:.5f}'.format(tf - ti))
+                            print('Transitioning {} -> {}'.format(self.state(), next_state))
+                        if self._response is not None:
+                            self._response = None
+                        if '__response_prefix__' in self.vars() and self.vars()['__response_prefix__'] != 'None':
+                            response = self.vars()['__response_prefix__'] + ' ' + response
+                            self.vars()['__response_prefix__'] = 'None'
+                        return response, next_state
+                else:
+                    return self.system_transition(target, debugging=debugging)
+            else:
+                if self._default_state is not None:
+                    self.set_state(self._default_state)
+                    if debugging:
+                        print('No valid system transitions found, going to default state...')
+                    return self.system_transition(self.state(), debugging=debugging)
+                raise AssertionError('dialogue flow system transition found no valid options')
+
 
     def user_transition(self, natural_language: str, state: Union[Enum, str, tuple], debugging=False):
         """
@@ -364,17 +464,25 @@ class DialogueFlow:
         :return: the successor state representing the highest score user transition
                  that matches natural_language, or None if none match
         """
+        state = module_state(state)
         self._error_transitioned = False
         ti = time()
         if state is None:
-            state = self._state
+            state = self.state()
         else:
             state = State(state)
         transition_options = []
+        if '__transition__' in self.vars() and self.vars()['__transition__']:
+            transition_options.append(
+                (self.vars()['__transition_score__'],
+                 ('__global__', self.vars()['__transition__'], Speaker.USER),
+                 self.vars()))
         ngrams = Ngrams(natural_language, n=10)
         self._gate_buffer.clear()
         for transition in self.transitions(state, Speaker.USER):
             self._potential_transition = transition
+            if not self._is_module and isinstance(transition[1], tuple):
+                continue
             t1 = time()
             if debugging:
                 print('Evaluating transition {}'.format(transition[:2]))
@@ -438,7 +546,7 @@ class DialogueFlow:
         if process_num == 1:
             for transition in self._graph.arcs():
                 data = self._graph.arc_data(*transition)
-                data['natex'].precache()
+                data['question_natex'].precache()
         else:
             transition_data_sets = []
             for i in range(process_num):
@@ -457,7 +565,7 @@ class DialogueFlow:
                 for j in range(len(result_list)):
                     parsed_tree = result_list[j]
                     t = t_list[j]
-                    t['natex']._compiler._parsed_tree = parsed_tree
+                    t['question_natex']._compiler._parsed_tree = parsed_tree
 
 
     def check(self, debugging=False):
@@ -486,13 +594,11 @@ class DialogueFlow:
 
     def add_user_transition(self, source: Union[Enum, str, tuple], target: Union[Enum, str, tuple],
                             natex_nlu: Union[str, NatexNLU, List[str]], **settings):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
         if self.has_transition(source, target, Speaker.USER):
-            if self.transition_global_nlu(source, target):
-                self.remove_transition(source, target, Speaker.USER)
-            else:
-                raise ValueError('user transition {} -> {} already exists'.format(source, target))
+            raise ValueError('user transition {} -> {} already exists'.format(source, target))
         natex_nlu = NatexNLU(natex_nlu, macros=self._macros)
         if not self.has_state(source):
             self.add_state(source)
@@ -510,6 +616,7 @@ class DialogueFlow:
 
     def add_system_transition(self, source: Union[Enum, str, tuple], target: Union[Enum, str, tuple],
                               natex_nlg: Union[str, NatexNLG, List[str]], **settings):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
         if self.has_transition(source, target, Speaker.SYSTEM):
@@ -530,23 +637,23 @@ class DialogueFlow:
             self.set_transition_natex(source, target, Speaker.SYSTEM, prepend + natex)
 
     def add_state(self, state: Union[Enum, str, tuple], error_successor: Union[Union[Enum, str, tuple], None] =None, **settings):
+        state = module_state(state)
         state = State(state)
         if self.has_state(state):
             raise ValueError('state {} already exists'.format(state))
-        state_settings = Settings(user_multi_hop=False, system_multi_hop=False, global_nlu=None, memory=10)
+        state_settings = Settings(user_multi_hop=False, system_multi_hop=False, switch=False, memory=10)
         state_settings.update(**settings)
         self._graph.add_node(state)
         self.update_state_settings(state, **state_settings)
         if error_successor is not None:
             error_successor = State(error_successor)
             self.set_error_successor(state, error_successor)
-        for global_target in self._global_states:
-            if not self.has_transition(state, global_target, Speaker.USER):
-                self.add_user_transition(state, global_target, self.state_settings(global_target).global_nlu)
-                self.set_transition_global(state, global_target, is_global=True)
+
+
     # MID LEVEL
 
     def take_transition(self, target):
+        target = module_state(target)
         if self.speaker() is Speaker.SYSTEM:
             transition = (self.state(), target, self.speaker())
             self.state_settings(self.state()).memory.add(transition)
@@ -561,75 +668,64 @@ class DialogueFlow:
     # LOW LEVEL: PROPERTIES, GETTERS, SETTERS
 
     def transition_natex(self, source: Union[Enum, str, tuple], target: Union[Enum, str, tuple], speaker: Enum):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
-        return self._graph.arc_data(source, target, speaker)['natex']
+        return self._graph.arc_data(source, target, speaker)['question_natex']
 
     def set_transition_natex(self, source, target, speaker, natex):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
-        self._graph.arc_data(source, target, speaker)['natex'] = natex
+        self._graph.arc_data(source, target, speaker)['question_natex'] = natex
 
     def transition_settings(self, source: Union[Enum, str, tuple], target: Union[Enum, str, tuple], speaker: Enum):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
         return self._graph.arc_data(source, target, speaker)['settings']
 
     def set_transition_settings(self, source, target, speaker, settings):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
         self._graph.arc_data(source, target, speaker)['settings'] = settings
 
     def update_transition_settings(self, source, target, speaker, **settings):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
         self.transition_settings(source, target, speaker).update(**settings)
 
-    def transition_global_nlu(self, source: Union[Enum, str, tuple], target: Union[Enum, str, tuple]):
-        source = State(source)
-        target = State(target)
-        if self.has_transition(source, target, Speaker.USER) \
-        and 'global' in self._graph.arc_data(source, target, Speaker.USER):
-            return self._graph.arc_data(source, target, Speaker.USER)['global']
-        return False
-
-    def set_transition_global(self, source, target, is_global):
-        source = State(source)
-        target = State(target)
-        self._graph.arc_data(source, target, Speaker.USER)['global'] = is_global
-        self.update_transition_settings(source, target, Speaker.USER, score=0.5)
-
     def state_settings(self, state):
+        state = module_state(state)
         state = State(state)
         return self._graph.data(state)['settings']
 
-    def _update_global_states(self, state):
+    def add_global_nlu(self, state, nlu, score=0.5):
+        state = module_state(state)
         state = State(state)
-        global_nlu = self.state_settings(state).global_nlu
-        if global_nlu:
-            self._global_states.add(state)
-            for source in self.states():
-                if source is not state and not self.has_transition(source, state, Speaker.USER):
-                    self.add_user_transition(source, state, global_nlu)
-                    self.set_transition_global(source, state, is_global=True)
-        else:
-            if state in self._global_states:
-                self._global_states.remove(state)
-                for source in self.states():
-                    for transition in list(self.transitions(source)):
-                        if self.transition_global_nlu(*transition):
-                            self.remove_transition(*transition)
+        if not self.has_state(state):
+            self.add_state(state)
+        if isinstance(state, tuple):
+            state = ':'.join(state)
+        if isinstance(nlu, list) or isinstance(nlu, set):
+            nlu = '{' + ', '.join(nlu) + '}'
+        self._rules.add('{} ({})'.format(nlu, score), '#TRANSITION({})'.format(state, score))
 
     def update_state_settings(self, state, **settings):
+        state = module_state(state)
         state = State(state)
         if 'settings' not in self._graph.data(state):
             self._graph.data(state)['settings'] = Settings()
         if 'memory' in settings:
             settings['memory'] = Memory(settings['memory'])
+        if 'global_nlu' in settings:
+            self.add_global_nlu(state, settings['global_nlu'])
         self.state_settings(state).update(**settings)
-        self._update_global_states(state)
 
     def remove_transition(self, source, target, speaker):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
         MapMultidigraph.remove_arc(self.graph(), source, target, speaker)
@@ -638,17 +734,20 @@ class DialogueFlow:
         return self.graph().nodes()
 
     def state(self):
-        return self._state
+        return self._vars['__state__']
 
     def set_state(self, state: Union[Enum, str, tuple]):
+        state = module_state(state)
         state = State(state)
-        self._state = state
+        self._vars['__state__'] = state
 
     def has_state(self, state):
+        state = module_state(state)
         state = State(state)
         return self._graph.has_node(state)
 
     def error_successor(self, state):
+        state = module_state(state)
         state = State(state)
         data = self._graph.data(state)
         if 'error' in data:
@@ -657,6 +756,7 @@ class DialogueFlow:
             return None
 
     def set_error_successor(self, state, error_successor):
+        state, error_successor = module_source_target(state, error_successor)
         state = State(state)
         error_successor = State(error_successor)
         self._graph.data(state)['error'] = error_successor
@@ -684,6 +784,7 @@ class DialogueFlow:
         :param speaker: optionally, filter returned transitions by speaker
         :return: a generator over (source, target, speaker) 3-tuples
         """
+        source_state = module_state(source_state)
         source_state = State(source_state)
         if speaker is None:
             yield from self._graph.arcs_out(source_state)
@@ -693,11 +794,13 @@ class DialogueFlow:
             return
 
     def has_transition(self, source, target, speaker):
+        source, target = module_source_target(source, target)
         source = State(source)
         target = State(target)
         return self._graph.has_arc(source, target, speaker)
 
     def incoming_transitions(self, target_state):
+        target_state = module_state(target_state)
         target_state = State(target_state)
         yield from self._graph.arcs_in(target_state)
 
@@ -708,24 +811,24 @@ class DialogueFlow:
             self.set_speaker(Speaker.USER)
 
     def reset(self):
-        self._state = self._initial_state
         self._speaker = self._initial_speaker
-        self._vars = {}
+        self._vars = HashableDict()
+        self.set_state(self._initial_state)
+        self._rules.set_vars(self._vars)
         self._gates = defaultdict(set)
         for state in self.graph().nodes():
             self.state_settings(state).memory.clear()
 
-    def update_vars(self, vars: HashableDict):
-        if not isinstance(vars, HashableDict):
-            vars = HashableDict(vars)
-        for k in vars.altered():
+    def update_vars(self, variables: HashableDict):
+        if not isinstance(variables, HashableDict):
+            variables = HashableDict(variables)
+        for k in variables.altered():
             if k in self._var_dependencies:
                 dependencies = self._var_dependencies[k]
                 for dependency in dependencies:
                     if dependency in self._vars:
                         self._vars[dependency] = None
-
-        self._vars.update({k: vars[k] for k in vars.altered()})
+        self._vars.update({k: variables[k] for k in variables.altered()})
 
     def potential_transition(self):
         return self._potential_transition
@@ -749,6 +852,7 @@ class DialogueFlow:
         return self._var_dependencies
 
     def set_state_prepend(self, state, prepend):
+        state = module_state(state)
         self._prepends[state] = prepend
         if self.has_state(state):
             for transition in self._graph.arcs_in(state):
@@ -776,12 +880,17 @@ class DialogueFlow:
         self._rules.add(precondition, postcondition)
 
     def state_update(self, user_input, debugging=False):
-        vars = HashableDict(self.vars())
-        self._rules.set_vars(vars)
         result = self._rules.update(user_input, debugging)
         if result is not None:
             response, score = result
-            self._response = response, vars, self._state, score
+            self._response = response, self._vars, self.state(), score
+            self.set_speaker(Speaker.SYSTEM)
 
     def knowledge_base(self):
         return self._kb
+
+    def set_is_module(self):
+        self._is_module = True
+
+    def is_switch(self, state):
+        return self.state_settings(state)['switch']
