@@ -4,7 +4,7 @@ import json, os
 import random
 
 # Xiangjue Dong
-# 2/18/2020
+# 3/21/2020
 
 # States are typically represented as an enum
 class State(Enum):
@@ -13,11 +13,9 @@ class State(Enum):
     START_PET = 1
     END = 2
     PETS_Y = 3
-    PETS_Y_1 = 4
-    PETS_Y_2 = 5
-    ASK_PETS = 6
-    ASK_PETS_Y = 7
-    ASK_PETS_N = 8
+    ASK_PETS = 4
+    ASK_PETS_Y = 5
+    ASK_PETS_N = 6
 
     FIRST_PET = 10
     FIRST_PET_DOG = 11
@@ -133,6 +131,11 @@ class State(Enum):
     ANOTHER_CAT_BREED_N = 152
     ANOTHER_CAT_BREED_DETAIL = 153
 
+    PETS_PROS = 160
+    PETS_PROS_DB = 161
+    PETS_PROS_N = 162
+    PETS_PROS_UNKNOWN = 163
+
 
 class CATCH(Macro):
     """Catch user utterance with list.
@@ -150,7 +153,6 @@ class CATCH(Macro):
         return ngrams & self.list
 
 
-
 class RANDOM(Macro):
     """Generate random information from database.
 
@@ -159,71 +161,31 @@ class RANDOM(Macro):
         db_keys: Keys of database.
     """
 
-    def __init__(self, path):
-        """Inits RANDOM with path and db_keys"""
-        self.path = path
-        self.db_keys = []
-        with open(self.path, 'r') as f:
-            self.db = json.load(f)
-
-    def run(self, ngrams, vars, args):
-        """Performs operation"""
-        if len(args) == 0:
-            if len(self.db_keys) == 0:
-                self.db_keys = list(self.db.keys())
-                key = random.choice(self.db_keys)
-                self.db_keys.remove(key)
-                return key
-            else:
-                key = random.choice(self.db_keys)
-                self.db_keys.remove(key)
-                return key
-
-        if len(args) == 1:
-            return self.db[vars[args[0]]]
-
-
-class RANDOM_BREED(Macro):
-    """Generate random information from database.
-
-    Attributes:
-        path: Path of database.
-        db_keys: Keys of database.
-    """
-
     def __init__(self, path):
         """Inits RANDOM_BREED with path and db_keys"""
         self.path = path
-        self.db_keys = []
         with open(self.path, 'r') as f:
             self.db = json.load(f)
 
     def run(self, ngrams, vars, args):
         """Performs operation"""
+        name = 'db_keys'+self.path
         if len(args) == 0:
-            if len(self.db_keys) == 0:
-                self.db_keys = list(self.db.keys())
-                key = random.choice(self.db_keys)
-                self.db_keys.remove(key)
-                return key
-            else:
-                key = random.choice(self.db_keys)
-                self.db_keys.remove(key)
-                return key
+            if vars.get(name) is None or len(vars[name]) == 0:
+                vars[name] = list(self.db.keys())
+            key = random.choice(vars[name])
+            vars[name].remove(key)
+            return key
+
         elif len(args) == 1:
-            if len(self.db_keys) == 0:
-                self.db_keys = list(self.db.keys())
-                if vars[args[0]] in self.db_keys:
-                    self.db_keys.remove(vars[args[0]])
-                key = random.choice(self.db_keys)
-                self.db_keys.remove(key)
-                return key
-            else:
-                if vars[args[0]] in self.db_keys:
-                    self.db_keys.remove(vars[args[0]])
-                key = random.choice(self.db_keys)
-                self.db_keys.remove(key)
-                return key
+            if vars.get(name) is None or len(vars[name]) <= 1:
+                vars[name] = list(self.db.keys())
+            if vars[args[0]] in vars[name]:
+                vars[name].remove(vars[args[0]])
+            key = random.choice(vars[name])
+            vars[name].remove(key)
+            return key
+
         elif len(args) == 2:
             return self.db[vars[args[1]]]
 
@@ -241,6 +203,7 @@ class EMORA(Macro):
             response = "my favorite pet is a german shepherd dog."
         else:
             response = "my favorite pet is a german shepherd dog."
+
         return response
 
 
@@ -257,18 +220,16 @@ class INTERESTING(Macro):
         self.path = path
         with open(self.path, 'r') as f:
             self.db = json.load(f)
-        self.db_backup = self.db.copy()
 
     def run(self, ngrams, vars, args):
         """Performs operation"""
-        length = len(self.db)
-        if length > 2:
-            key = random.randrange(length)
-            response = self.db.pop(key)
-        else:
-            self.db = self.db_backup.copy()
-            key = random.randrange(length)
-            response = self.db.pop(key)
+        db_backup = self.db.copy()
+        name = 'db' + self.path
+        if vars.get(name) is None or len(vars[name]) == 0:
+            vars[name] = db_backup
+        response = random.choice(vars[name])
+        vars[name].remove(response)
+
         return response
 
 
@@ -311,16 +272,8 @@ class BREED(Macro):
 # Variables
 TRANSITION_OUT = ["movies", "movie", "music", "sports", "sport", "travel"]
 NULL = "NULL TRANSITION"
-PET_TYPE = {"dog", "dogs", "cat", "cats", "pet", "pets"}
-PET_TYPE_OTHER = {"alpaca","alpacas", "camel", "camels", "cattle", "cattles", "skunk", "skunks", "donkey",
-            "donkeys", "ferret", "ferrets", "goat", "goats", "hedgehog", "hedgehogs", "horse", "horses", "llama", "llamas", "pig",
-            "pigs", "rabbit", "rabbits", "red fox", "red foxes", "rodent", "rodents", "sheep", "sugar gliders", "bird", "birds", "fish",
-            "fishes", "arthropod", "arthropods"}
-PET_TYPE_TOTAL = {"dog", "dogs", "puppies", "puppy", "kitties", "kitty", "cat", "cats", "pet", "pets", "animal", "animals","alpaca","alpacas", "camel", "camels", "cattle", "cattles", "skunk", "skunks", "donkey",
-            "donkeys", "ferret", "ferrets", "goat", "goats", "hedgehog", "hedgehogs", "horse", "horses", "llama", "llamas", "pig",
-            "pigs", "rabbit", "rabbits", "red fox", "red foxes", "rodent", "rodents", "sheep", "sugar gliders", "bird", "birds", "fish",
-            "fishes", "arthropod", "arthropods"}
-NAME = {"max","charlie","cooper","buddy","rocky","milo","jack","bear","duke","teddy","oliver","bentley","tucker","beau","leo","toby","jax","zeus","winston","blue","finn","louie","ollie","murphy","gus","moose",
+PET_TYPE = {"dog", "dogs", "puppies", "puppy", "kitties", "kitty", "cat", "cats", "pet", "pets", "animal", "animals"}
+NAME = {"alex","max","charlie","cooper","buddy","rocky","milo","jack","bear","duke","teddy","oliver","bentley","tucker","beau","leo","toby","jax","zeus","winston","blue","finn","louie","ollie","murphy","gus","moose",
         "jake","loki","dexter","hank","bruno","apollo","buster","thor","bailey","gunnar","lucky","diesel","harley","henry","koda","jackson","riley", "ace", "oscar","chewy","bandit","baxter","scout","jasper",
         "maverick","sam","cody","gizmo","shadow","simba","rex","brody","tank","marley","otis","remi","remy","roscoe","rocco","sammy","cash","boomer","prince","benji","benny","archie","chance","ranger","ziggy",
         "luke","george","oreo","hunter","rusty","king","odin","coco","frankie","tyson","chase","theo","romeo","bruce","rudy","zeke","kobe","peanut","joey","oakley","chico","mac","walter","brutus","samson","bella",
@@ -328,8 +281,8 @@ NAME = {"max","charlie","cooper","buddy","rocky","milo","jack","bear","duke","te
         "lulu","pepper","willow","riley","millie","harley","sasha","lady","izzy","layla","charlie","dixie","maya","annie","kona","hazel","winnie","olive","princess","emma","athena","nova","belle","honey","ella",
         "marley","cookie","maddie","remi","remy","phoebe","scout","minnie","dakota","holly","angel","josie","leia","harper","ava","missy","mila","sugar","shelby","poppy","blue","mocha","cleo","penelope","ivy","peanut",
         "fiona","xena","gigi","sandy","bonnie","jasmine","baby","macy","paisley","shadow","koda","pearl","skye","delilah","nina","trixie","charlotte","aspen","arya","diamond","georgia","dolly"}
-YES = {"yes", "yea", "yup", "yep", "i do", "yeah", "a little", "sure", "of course", "i have", "i am", "sometimes", "too", "as well", "also", "agree","good", "keep"}
-NO = {"no", "nope", "dont", "i dont"}
+YES = {"yes", "yea", "yup", "yep", "i do", "yeah", "a little", "sure", "of course", "i have", "i am", "sometimes", "too", "as well", "also", "agree","good", "keep","why not", "ok", "okay", "fine", "continue", "go on"}
+NO = {"no", "nope", "dont", "nothing"}
 
 
 # Functions
@@ -344,6 +297,8 @@ dog_interesting = os.path.join('modules','dog_interesting_database.json')
 other_interesting = os.path.join('modules','other_interesting_database.json')
 cat_food = os.path.join('modules','cat_food_database.json')
 dog_food = os.path.join('modules','dog_food_database.json')
+pet_pros = os.path.join('modules','pets_pros.json')
+
 macros = {
     'DOG_BREED': BREED(dog_breed),
     'DOG_BREED_DESC': BREED_DESC(dog_breed),
@@ -352,30 +307,30 @@ macros = {
     'OTHER_BREED': BREED(other_breed),
     'OTHER_BREED_DESC': BREED_DESC(other_breed),
     'CATCH_PET_TYPE':CATCH(PET_TYPE),
-    'CATCH_PET_TYPE_OTHER':CATCH(PET_TYPE_OTHER),
-    'CATCH_PET_TYPE_TOTAL':CATCH(PET_TYPE_TOTAL),
     'CATCH_YES':CATCH(YES),
     'CATCH_NO':CATCH(NO),
     'EMORA':EMORA(),
     'DOG_INTERESTING':INTERESTING(dog_interesting),
     'CAT_INTERESTING':INTERESTING(cat_interesting),
     'OTHER_INTERESTING':INTERESTING(other_interesting),
-    'DOG_MOVIE':RANDOM_BREED(dog_movie),
-    'CAT_MOVIE':RANDOM_BREED(cat_movie),
-    'OTHER_MOVIE':RANDOM_BREED(other_movie),
+    'DOG_MOVIE':RANDOM(dog_movie),
+    'CAT_MOVIE':RANDOM(cat_movie),
+    'OTHER_MOVIE':RANDOM(other_movie),
     'DOG_MOVIE_DESC': BREED_DESC(dog_movie),
     'CAT_MOVIE_DESC': BREED_DESC(cat_movie),
     'OTHER_MOVIE_DESC':BREED_DESC(other_movie),
-    'DOG_RANDOM_BREED': RANDOM_BREED(dog_breed),
-    'CAT_RANDOM_BREED': RANDOM_BREED(cat_breed),
-    'OTHER_RANDOM_BREED': RANDOM_BREED(other_breed),
-    'DOG_FOOD': RANDOM_BREED(dog_food),
-    'CAT_FOOD': RANDOM_BREED(cat_food),
+    'DOG_RANDOM_BREED': RANDOM(dog_breed),
+    'CAT_RANDOM_BREED': RANDOM(cat_breed),
+    'OTHER_RANDOM_BREED': RANDOM(other_breed),
+    'DOG_FOOD': RANDOM(dog_food),
+    'CAT_FOOD': RANDOM(cat_food),
     'CATCH_DOG_FOOD':BREED(dog_food),
     'CATCH_CAT_FOOD':BREED(cat_food),
     'CATCH_DOG_FOOD_DESC': BREED_DESC(dog_food),
     'CATCH_CAT_FOOD_DESC': BREED_DESC(cat_food),
-    'NAME':CATCH(NAME)
+    'NAME':CATCH(NAME),
+    'CATCH_PROS':BREED(pet_pros),
+    'CATCH_PROS_DESC':BREED_DESC(pet_pros)
 }
 
 ###################### Initialization Part ####################################################################################################################
@@ -386,7 +341,7 @@ df = DialogueFlow(State.START, initial_speaker=DialogueFlow.Speaker.USER, macros
 # For dialogue manager initialization
 # test
 # df.add_user_transition(State.START, State.START_PET, 'test')
-df.add_system_transition(State.START_PET, State.ASK_PETS, '"ok, i was curious about this. Do you have any pets at home? like a"$breed={dog, cat}"?"')
+df.add_system_transition(State.START_PET, State.ASK_PETS, '"So, do you have any pets at home? like a"$breed={dog, cat}"?"')
 
 # df.add_user_transition(State.START, State.ASK_PETS_Y, '[{pet, pets}]')
 # df.add_user_transition(State.START, State.ASK_PETS_Y, '[![!i have] {pet, pets}]')
@@ -394,9 +349,9 @@ df.add_system_transition(State.START_PET, State.ASK_PETS, '"ok, i was curious ab
 # df.add_user_transition(State.START, State.ASK_PETS_N, '[![!i, dont, have] {dog, dogs}]') # This didn't work well with next line at the same time
 
 # User Turn
-df.add_user_transition(State.START, State.PETS_Y, '[$breed=#CATCH_PET_TYPE_TOTAL()]')
-df.add_user_transition(State.START, State.PETS_Y_1, '[$breed=#DOG_BREED()]')
-df.add_user_transition(State.START, State.PETS_Y_2, '[$breed=#CAT_BREED()]')
+df.add_user_transition(State.START, State.PETS_Y, '[$breed={#OTHER_BREED(),#DOG_BREED(),#CAT_BREED(),#CATCH_PET_TYPE()}]')
+# df.add_user_transition(State.START, State.PETS_Y_1, '[$breed=#DOG_BREED()]')
+# df.add_user_transition(State.START, State.PETS_Y_2, '[$breed=#CAT_BREED()]')
 
 # Error Transition
 df.set_error_successor(State.START, State.START)
@@ -409,10 +364,10 @@ df.add_system_transition(State.START, State.START, NULL)
 ###################### Pet Opening Part ###########################################################################################################################
 # System Turn
 df.add_system_transition(State.PETS_Y, State.ASK_PETS, '"That\'s great! I love" $breed "! actually, " #EMORA(breed) "Do you have any pets at your home now?"')
-df.add_system_transition(State.PETS_Y_1, State.ASK_PETS, '"That\'s cool! I love" $breed "! actually, " #EMORA(breed) "Do you have any pets at your home now?"')
-df.add_system_transition(State.PETS_Y_2, State.ASK_PETS, '"Sounds cool! I love" $breed "! actually, " #EMORA(breed) "Do you have any pets at your home now?"')
+# df.add_system_transition(State.PETS_Y_1, State.ASK_PETS, '"That\'s cool! I love" $breed "! actually, " #EMORA(breed) "Do you have any pets at your home now?"')
+# df.add_system_transition(State.PETS_Y_2, State.ASK_PETS, '"Sounds cool! I love" $breed "! actually, " #EMORA(breed) "Do you have any pets at your home now?"')
 
-df.add_user_transition(State.ASK_PETS, State.ASK_PETS_Y, '[![{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep}] #NOT(no, dont, nope, dog, dogs, cat, cats, pet, pets)]')
+df.add_user_transition(State.ASK_PETS, State.ASK_PETS_Y, '[![{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why}] #NOT(no, dont, nope, dog, dogs, cat, cats, pet, pets)]')
 
 # User Turn
 # df.add_user_transition(State.ASK_PETS, State.ASK_PETS_N, '[[{no, nope, dont}] -{know, idea}]')
@@ -426,7 +381,8 @@ df.add_user_transition(State.ASK_PETS, State.FIRST_PET_CAT_BREED, '[$breed=#CAT_
 df.add_user_transition(State.ASK_PETS, State.FAVORITE_PET_DOG_DONTKNOW, '[{i dont know, i have no idea, who knows}]')
 
 # Error transition
-df.set_error_successor(State.ASK_PETS, State.FIRST_PET_OTHER_UNKNOWN)
+# df.set_error_successor(State.ASK_PETS, State.FIRST_PET_OTHER_UNKNOWN)
+df.set_error_successor(State.ASK_PETS, State.NO_PETS_Y)
 
 # df.add_system_transition(State.ASK_PETS_UNKNOWN, State.ASK_PETS_UNKNOWN_ANS, '"Sorry, I was sleeping just now. What did you mean? If you mean you have a pet, what is it? If you mean you don\'t have a pet, would you like to know something about the pets?"')
 # df.add_user_transition(State.ASK_PETS_UNKNOWN_ANS, State.FIRST_PET_DOG, '[$breed=#DOG_BREED()]')
@@ -444,18 +400,11 @@ df.add_system_transition(State.ASK_PETS_Y, State.ASK_PETS, '"Cool! What is it? A
 # df.add_system_transition(State.FIRST_PET_UNKNOWN, State.FIRST_PET_NAME, '"Sorry, I\'m not quite familiar with this animal. But I would like to talk about it with you. Can you tell me more information of it? it must have a cute name. What is its name?"')
 
 # System Turn
-df.add_system_transition(State.ASK_PETS_N, State.NO_PETS, '"Me neither. Because my dad doesn\'t let me keep a dog. then, Would you consider getting a pet? i mean, after all, they are so cute."')
+df.add_system_transition(State.ASK_PETS_N, State.NO_PETS, '"You don\'t have any pets? '
+                                                          'Well, would you consider getting one? '
+                                                          'They are pretty cute."')
 
-# User Turn - Bad Taste
-df.add_user_transition(State.NO_PETS, State.NO_PETS_DAD, '[{dad, daddy, father, papa, parent, parents}]')
-df.add_system_transition(State.NO_PETS_DAD, State.NO_PETS_DAD_ANS, '"Interesting question. My dad is my grandpa\'s son. So, Would you consider getting a pet?"')
-df.add_user_transition(State.NO_PETS_DAD_ANS, State.NO_PETS_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, good, keep, why not}]')
-df.add_user_transition(State.NO_PETS_DAD_ANS, State.NO_PETS_N, '[{no, nope, dont, nothing}]')
-
-df.add_user_transition(State.NO_PETS_DAD_ANS, State.NO_PETS_DAD_ANS_ANS, '[{grandpa, grandfather}]')
-df.add_system_transition(State.NO_PETS_DAD_ANS_ANS, State.NO_PETS, '"Interesting question. My grandpa is my father\'s father. So, Would you consider getting a pet?"')
-
-df.add_user_transition(State.NO_PETS, State.NO_PETS_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, good, keep, why not}]')
+df.add_user_transition(State.NO_PETS, State.NO_PETS_Y, '[#CATCH_YES()]')
 df.add_user_transition(State.NO_PETS, State.NO_PETS_N, '[!-{why}[{no, nope, dont, nothing, not}]]')
 
 
@@ -472,7 +421,7 @@ df.add_system_transition(State.NO_PETS_UNKNOWN, State.ANOTHER_DOG_BREED, '"I see
 
 #################### Transition Part ###########################################################################################################################
 # System Turn
-df.add_system_transition(State.NO_PETS_N, State.ANOTHER_DOG_BREED, '"Oh ok, that\'s fine. it is indeed a big decision to introduce new members into your family or life. I know a lot about dogs. Although you won\'t keep a dog, Would you like to know some information about dog breed, like" $another_breed={#DOG_RANDOM_BREED(breed)} "?"')
+df.add_system_transition(State.NO_PETS_N, State.ANOTHER_DOG_BREED, '"Oh ok, that\'s fine. It is indeed a big decision to introduce new members into your family or life. I know a lot about dogs. Although you won\'t keep a dog, would you like to know some information about dog breed, like" $another_breed={#DOG_RANDOM_BREED(breed)} "?"')
 
 # User Turn
 # df.add_user_transition(State.NO_NO_PETS, State.FAVORITE_PET_DOG, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay}]')
@@ -490,9 +439,9 @@ df.add_system_transition(State.NO_PETS_N, State.ANOTHER_DOG_BREED, '"Oh ok, that
 
 ################## First Pet Part ################################################################################################################################
 # System Turn
-df.add_system_transition(State.FIRST_PET_DOG, State.FIRST_PET_DOG_ANS, '{Cool,Wonderful,Wow, Awesome}"! that\'s great! I like dogs! What is its breed?"')
-df.add_system_transition(State.FIRST_PET_CAT, State.FIRST_PET_CAT_ANS, '{Cool,Wonderful,Wow, Awesome}"! that\'s great! I like cats! What is its breed?"')
-df.add_system_transition(State.FIRST_PET_OTHER, State.FIRST_PET_OTHER_ANS, '{Cool,Wonderful,Wow, Awesome}"! that\'s interesting! What is it? A fish? a bird?"')
+df.add_system_transition(State.FIRST_PET_DOG, State.FIRST_PET_DOG_ANS, '{Cool,Wonderful,Wow, Awesome}"! That\'s great! I like dogs! What is its breed?"')
+df.add_system_transition(State.FIRST_PET_CAT, State.FIRST_PET_CAT_ANS, '{Cool,Wonderful,Wow, Awesome}"! That\'s great! I like cats! What is its breed?"')
+df.add_system_transition(State.FIRST_PET_OTHER, State.FIRST_PET_OTHER_ANS, '{Cool,Wonderful,Wow, Awesome}"! That\'s interesting! What is it? A fish? a bird?"')
 
 # User Turn
 df.add_user_transition(State.FIRST_PET_DOG_ANS, State.FIRST_PET_DOG_BREED, '[$breed=#DOG_BREED()]')
@@ -507,7 +456,7 @@ df.set_error_successor(State.FIRST_PET_OTHER_ANS, State.FIRST_PET_OTHER_UNKNOWN)
 # System Turn - Ask Name
 df.add_system_transition(State.FIRST_PET_DOG_BREED, State.FIRST_PET_DOG_NAME, '{Cool,Wonderful,Wow, Awesome}"! Sounds interesting!" #DOG_BREED_DESC(breed) "I guess it must have a cute name. What is its name?"')
 df.add_system_transition(State.FIRST_PET_CAT_BREED, State.FIRST_PET_CAT_NAME, '{Cool,Wonderful,Wow, Awesome}"! Sounds interesting!" #CAT_BREED_DESC(breed) "I guess it must have a cute name. What is its name?"')
-df.add_system_transition(State.FIRST_PET_OTHER_BREED, State.FIRST_PET_OTHER_NAME, '"{Cool,Wonderful,Wow, Awesome}! Sounds interesting. " #OTHER_BREED_DESC(breed)"I guess it must have a cute name. What is its name?"')
+df.add_system_transition(State.FIRST_PET_OTHER_BREED, State.FIRST_PET_OTHER_NAME, '{Cool,Wonderful,Wow, Awesome}"! Sounds interesting. " #OTHER_BREED_DESC(breed)"I guess it must have a cute name. What is its name?"')
 
 df.add_system_transition(State.FIRST_PET_DOG_UNKNOWN, State.FIRST_PET_DOG_NAME, '{Cool,Wonderful,Wow, Awesome}"! Sounds interesting. Although I am not quite familiar with this dog breed, I guess it must have a cute name. What is its name?"')
 df.add_system_transition(State.FIRST_PET_CAT_UNKNOWN, State.FIRST_PET_CAT_NAME, '{Cool,Wonderful,Wow, Awesome}"! Sounds interesting. Although I am not quite familiar with this cat breed, I guess it must have a cute name. What is its name?"')
@@ -535,23 +484,23 @@ df.add_system_transition(State.FIRST_PET_OTHER_NAME_UNKNOWN, State.FIRST_PET_DOG
 
 # User Turn & Error Transition - Answer Food
 # User Turn
-df.add_user_transition(State.FIRST_PET_DOG_FOOD, State.FIRST_PET_DOG_FOOD_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not, why}]')
-df.add_user_transition(State.FIRST_PET_DOG_FOOD, State.FIRST_PET_DOG_FOOD_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.FIRST_PET_DOG_FOOD, State.FIRST_PET_DOG_FOOD_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.FIRST_PET_DOG_FOOD, State.FIRST_PET_DOG_FOOD_N, '[#CATCH_NO()]')
 
-df.add_user_transition(State.FIRST_PET_CAT_FOOD, State.FIRST_PET_CAT_FOOD_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not, why}]')
-df.add_user_transition(State.FIRST_PET_CAT_FOOD, State.FIRST_PET_CAT_FOOD_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.FIRST_PET_CAT_FOOD, State.FIRST_PET_CAT_FOOD_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.FIRST_PET_CAT_FOOD, State.FIRST_PET_CAT_FOOD_N, '[#CATCH_NO()]')
 
-df.add_user_transition(State.FIRST_PET_OTHER_FOOD, State.FIRST_PET_DOG_FOOD_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not, why}]')
-df.add_user_transition(State.FIRST_PET_OTHER_FOOD, State.FIRST_PET_DOG_FOOD_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.FIRST_PET_OTHER_FOOD, State.FIRST_PET_DOG_FOOD_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.FIRST_PET_OTHER_FOOD, State.FIRST_PET_DOG_FOOD_N, '[#CATCH_NO()]')
 
 df.add_user_transition(State.FIRST_PET_DOG_FOOD, State.FIRST_PET_DOG_FOOD_USER, '[$food=#CATCH_DOG_FOOD()]')
 df.add_user_transition(State.FIRST_PET_CAT_FOOD, State.FIRST_PET_CAT_FOOD_USER, '[$food=#CATCH_CAT_FOOD()]')
 
-df.add_system_transition(State.FIRST_PET_DOG_FOOD_USER, State.FIRST_PET_DOG_FOOD, '"My pleasure." #CATCH_DOG_FOOD_DESC($food) "Would you like to know about a different food too, like" $food={#DOG_FOOD(food)} "?"')
-df.add_system_transition(State.FIRST_PET_CAT_FOOD_USER, State.FIRST_PET_CAT_FOOD, '"My pleasure." #CATCH_CAT_FOOD_DESC($food) "Would you like to know about a different food too, like" $food={#CAT_FOOD(food)} "?"')
+df.add_system_transition(State.FIRST_PET_DOG_FOOD_USER, State.FIRST_PET_DOG_FOOD, '"My pleasure." #CATCH_DOG_FOOD_DESC(food) "Would you like to know about a different food too, like" $food={#DOG_FOOD(food)} "?"')
+df.add_system_transition(State.FIRST_PET_CAT_FOOD_USER, State.FIRST_PET_CAT_FOOD, '"My pleasure." #CATCH_CAT_FOOD_DESC(food) "Would you like to know about a different food too, like" $food={#CAT_FOOD(food)} "?"')
 
-df.add_system_transition(State.FIRST_PET_DOG_FOOD_Y, State.FIRST_PET_DOG_FOOD, '"My pleasure." #CATCH_DOG_FOOD_DESC($food) "Would you like to know about a different food too, like" $food={#DOG_FOOD(food)} "?"')
-df.add_system_transition(State.FIRST_PET_CAT_FOOD_Y, State.FIRST_PET_CAT_FOOD, '"My pleasure." #CATCH_CAT_FOOD_DESC($food) "Would you like to know about a different food too, like" $food={#CAT_FOOD(food)} "?"')
+df.add_system_transition(State.FIRST_PET_DOG_FOOD_Y, State.FIRST_PET_DOG_FOOD, '{Nice choice,Good choice,My pleasure}"." #CATCH_DOG_FOOD_DESC(food) "Would you like to know about a different food too, like" $food={#DOG_FOOD(food)} "?"')
+df.add_system_transition(State.FIRST_PET_CAT_FOOD_Y, State.FIRST_PET_CAT_FOOD, '{Nice choice,Good choice,My pleasure}"." #CATCH_CAT_FOOD_DESC(food) "Would you like to know about a different food too, like" $food={#CAT_FOOD(food)} "?"')
 
 df.set_error_successor(State.FIRST_PET_DOG_FOOD, State.FIRST_PET_DOG_FOOD_N)
 df.set_error_successor(State.FIRST_PET_CAT_FOOD, State.FIRST_PET_DOG_FOOD_N)
@@ -607,11 +556,11 @@ df.add_system_transition(State.FAVORITE_PET_CAT_BREED, State.ANOTHER_CAT_BREED,
 df.add_system_transition(State.FAVORITE_PET_CAT_DONTKNOW, State.FAVORITE_PET_CAT_DONTKNOW_ANS, '"That\'s OK, that\'s why I\'m here. My favorite cat is" $another_breed="toyger" ".Would you like to know some information about this cat breed?"')
 
 # User Turn
-df.add_user_transition(State.FAVORITE_PET_DOG_DONTKNOW_ANS, State.ANOTHER_DOG_BREED, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.FAVORITE_PET_DOG_DONTKNOW_ANS, State.ANOTHER_DOG_BREED_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.FAVORITE_PET_DOG_DONTKNOW_ANS, State.ANOTHER_DOG_BREED, '[#CATCH_YES()]')
+df.add_user_transition(State.FAVORITE_PET_DOG_DONTKNOW_ANS, State.ANOTHER_DOG_BREED_N, '[#CATCH_NO()]')
 
-df.add_user_transition(State.FAVORITE_PET_CAT_DONTKNOW_ANS, State.ANOTHER_CAT_BREED, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.FAVORITE_PET_CAT_DONTKNOW_ANS, State.ANOTHER_CAT_BREED_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.FAVORITE_PET_CAT_DONTKNOW_ANS, State.ANOTHER_CAT_BREED, '[#CATCH_YES()]')
+df.add_user_transition(State.FAVORITE_PET_CAT_DONTKNOW_ANS, State.ANOTHER_CAT_BREED_N, '[#CATCH_NO()]')
 
 # Error Transition
 df.set_error_successor(State.FAVORITE_PET_OTHER_ANS, State.FAVORITE_PET_OTHER_UNKNOWN)
@@ -625,11 +574,11 @@ df.add_system_transition(State.FAVORITE_PET_OTHER_DONTKNOW, State.ANOTHER_DOG_BR
 
 ####################### Another Dog and Cat Breed Introduction Part ################################################################################################
 # User Turn
-df.add_user_transition(State.ANOTHER_DOG_BREED, State.ANOTHER_DOG_BREED_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.ANOTHER_DOG_BREED, State.ANOTHER_DOG_BREED_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.ANOTHER_DOG_BREED, State.ANOTHER_DOG_BREED_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.ANOTHER_DOG_BREED, State.ANOTHER_DOG_BREED_N, '[#CATCH_NO()]')
 
-df.add_user_transition(State.ANOTHER_CAT_BREED, State.ANOTHER_CAT_BREED_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.ANOTHER_CAT_BREED, State.ANOTHER_CAT_BREED_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.ANOTHER_CAT_BREED, State.ANOTHER_CAT_BREED_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.ANOTHER_CAT_BREED, State.ANOTHER_CAT_BREED_N, '[#CATCH_NO()]')
 
 # Error Transition
 df.set_error_successor(State.ANOTHER_DOG_BREED, State.ANOTHER_DOG_BREED_N)
@@ -643,24 +592,26 @@ df.add_system_transition(State.ANOTHER_CAT_BREED_Y, State.ANOTHER_CAT_BREED_DETA
 df.add_system_transition(State.ANOTHER_CAT_BREED_N, State.CAT_INTERESTING, '{Ok, Alright, Then, Well}", besides various cat breeds, I also know lots of interesting things about cats. Do you want to hear some?"')
 
 # User Turn
-df.add_user_transition(State.ANOTHER_DOG_BREED_DETAIL, State.ANOTHER_DOG_BREED_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.ANOTHER_DOG_BREED_DETAIL, State.ANOTHER_DOG_BREED_N, '[{no, nope, dont, nothing}]')
-df.add_user_transition(State.ANOTHER_DOG_BREED_DETAIL, State.ANOTHER_DOG_BREED_DETAIL, '[$another_breed=#DOG_BREED()]')
+df.add_user_transition(State.ANOTHER_DOG_BREED_DETAIL, State.ANOTHER_DOG_BREED_Y, '[{#CATCH_YES(),$another_breed=#DOG_BREED()}]')
+df.add_user_transition(State.ANOTHER_DOG_BREED_DETAIL, State.ANOTHER_DOG_BREED_N, '[#CATCH_NO()]')
 
-df.add_user_transition(State.ANOTHER_CAT_BREED_DETAIL, State.ANOTHER_CAT_BREED_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.ANOTHER_CAT_BREED_DETAIL, State.ANOTHER_CAT_BREED_N, '[{no, nope, dont, nothing}]')
-df.add_user_transition(State.ANOTHER_CAT_BREED_DETAIL, State.ANOTHER_CAT_BREED_DETAIL, '[$another_breed=#CAT_BREED()]')
+df.add_user_transition(State.ANOTHER_CAT_BREED_DETAIL, State.ANOTHER_CAT_BREED_Y, '[{#CATCH_YES(),$another_breed=#CAT_BREED()}]')
+df.add_user_transition(State.ANOTHER_CAT_BREED_DETAIL, State.ANOTHER_CAT_BREED_N, '[#CATCH_NO()]')
+
+# Error Transition
+df.set_error_successor(State.ANOTHER_DOG_BREED_DETAIL, State.ANOTHER_DOG_BREED_N)
+df.set_error_successor(State.ANOTHER_CAT_BREED_DETAIL, State.ANOTHER_CAT_BREED_N)
 
 ####################### Interesting Facts Part ######################################################################################################################
 # User Turn
-df.add_user_transition(State.DOG_INTERESTING, State.DOG_INTERESTING_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.DOG_INTERESTING, State.DOG_INTERESTING_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.DOG_INTERESTING, State.DOG_INTERESTING_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.DOG_INTERESTING, State.DOG_INTERESTING_N, '[#CATCH_NO()]')
 
-df.add_user_transition(State.CAT_INTERESTING, State.CAT_INTERESTING_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.CAT_INTERESTING, State.CAT_INTERESTING_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.CAT_INTERESTING, State.CAT_INTERESTING_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.CAT_INTERESTING, State.CAT_INTERESTING_N, '[#CATCH_NO()]')
 
-df.add_user_transition(State.OTHER_INTERESTING, State.OTHER_INTERESTING_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.OTHER_INTERESTING, State.OTHER_INTERESTING_N, '[{no, nope, dont, nothing}]')
+df.add_user_transition(State.OTHER_INTERESTING, State.OTHER_INTERESTING_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.OTHER_INTERESTING, State.OTHER_INTERESTING_N, '[#CATCH_NO()]')
 
 # Error Transition
 df.set_error_successor(State.DOG_INTERESTING, State.DOG_INTERESTING_N)
@@ -688,14 +639,14 @@ df.add_system_transition(State.CAT_INTERESTING_N, State.CAT_MOVIE, '{Ok, Alright
 df.add_system_transition(State.OTHER_INTERESTING_N, State.OTHER_MOVIE, '{Ok, Alright, Then, Well}", I also know lots of movies about animals. My favorite one is" $movie={#OTHER_MOVIE()} ".Would you like to know some details about it?"')
 
 # User Turn
-df.add_user_transition(State.DOG_MOVIE, State.DOG_MOVIE_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.DOG_MOVIE, State.DOG_MOVIE_N,'[{no, nope, dont, nothing}]')
+df.add_user_transition(State.DOG_MOVIE, State.DOG_MOVIE_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.DOG_MOVIE, State.DOG_MOVIE_N,'[#CATCH_NO()]')
 
-df.add_user_transition(State.CAT_MOVIE, State.CAT_MOVIE_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.CAT_MOVIE, State.CAT_MOVIE_N,'[{no, nope, dont, nothing}]')
+df.add_user_transition(State.CAT_MOVIE, State.CAT_MOVIE_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.CAT_MOVIE, State.CAT_MOVIE_N,'[#CATCH_NO()]')
 
-df.add_user_transition(State.OTHER_MOVIE, State.OTHER_MOVIE_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.OTHER_MOVIE, State.OTHER_MOVIE_N,'[{no, nope, dont, nothing}]')
+df.add_user_transition(State.OTHER_MOVIE, State.OTHER_MOVIE_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.OTHER_MOVIE, State.OTHER_MOVIE_N,'[#CATCH_NO()]')
 
 # Error Transition
 df.set_error_successor(State.DOG_MOVIE, State.DOG_MOVIE_N)
@@ -713,29 +664,43 @@ df.add_system_transition(State.OTHER_MOVIE_Y, State.OTHER_MOVIE_DETAIL, '{Nice c
 df.add_system_transition(State.OTHER_MOVIE_N, State.OTHER_MOVIE_DETAIL, '{Ok, Alright, Then, Well}", I also know some movies about other animals. Would you like to listen one, like" $movie={#OTHER_MOVIE(movie)} "?"')
 
 # User Turn
-df.add_user_transition(State.DOG_MOVIE_DETAIL, State.DOG_MOVIE_ANOTHER_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.DOG_MOVIE_DETAIL, State.DOG_MOVIE_ANOTHER_N,'[{no, nope, dont, nothing}]')
+df.add_user_transition(State.DOG_MOVIE_DETAIL, State.DOG_MOVIE_ANOTHER_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.DOG_MOVIE_DETAIL, State.DOG_MOVIE_ANOTHER_N,'[#CATCH_NO()]')
 df.set_error_successor(State.DOG_MOVIE_DETAIL, State.DOG_MOVIE_ANOTHER_N)
 
-df.add_user_transition(State.CAT_MOVIE_DETAIL, State.CAT_MOVIE_ANOTHER_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.CAT_MOVIE_DETAIL, State.CAT_MOVIE_ANOTHER_N,'[{no, nope, dont, nothing}]')
+df.add_user_transition(State.CAT_MOVIE_DETAIL, State.CAT_MOVIE_ANOTHER_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.CAT_MOVIE_DETAIL, State.CAT_MOVIE_ANOTHER_N,'[#CATCH_NO()]')
 df.set_error_successor(State.CAT_MOVIE_DETAIL, State.CAT_MOVIE_ANOTHER_N)
 
-df.add_user_transition(State.OTHER_MOVIE_DETAIL, State.OTHER_MOVIE_ANOTHER_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, okay, good, keep, why not}]')
-df.add_user_transition(State.OTHER_MOVIE_DETAIL, State.OTHER_MOVIE_ANOTHER_N,'[{no, nope, dont, nothing}]')
+df.add_user_transition(State.OTHER_MOVIE_DETAIL, State.OTHER_MOVIE_ANOTHER_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.OTHER_MOVIE_DETAIL, State.OTHER_MOVIE_ANOTHER_N,'[#CATCH_NO()]')
 df.set_error_successor(State.OTHER_MOVIE_DETAIL, State.OTHER_MOVIE_ANOTHER_N)
 
 # System Turn
 df.add_system_transition(State.DOG_MOVIE_ANOTHER_Y, State.DOG_MOVIE_DETAIL, '{Nice choice,Good choice, Cool,Wonderful,Wow, Awesome}"! An overview of" $movie "is that" #DOG_MOVIE_DESC(movie) "Would you like to know another one,like" $movie={#DOG_MOVIE(movie)} "?"')
-df.add_system_transition(State.DOG_MOVIE_ANOTHER_N, State.END, '')
+df.add_system_transition(State.DOG_MOVIE_ANOTHER_N, State.PETS_PROS, '{Ok, Alright, Then, Well}", I know lots of people have pets. What do you think are the benefits of keeping a pet?"')
 
 df.add_system_transition(State.CAT_MOVIE_ANOTHER_Y, State.CAT_MOVIE_DETAIL, '{Nice choice,Good choice, Cool,Wonderful,Wow, Awesome}"! An overview of" $movie"is that" #CAT_MOVIE_DESC(movie) "Would you like to know another one,like" $movie={#CAT_MOVIE(movie)} "?"')
-df.add_system_transition(State.CAT_MOVIE_ANOTHER_N, State.END, '')
+df.add_system_transition(State.CAT_MOVIE_ANOTHER_N, State.PETS_PROS, '{Ok, Alright, Then, Well}", I know lots of people have pets. What do you think are the benefits of keeping a pet?"')
 
 df.add_system_transition(State.OTHER_MOVIE_ANOTHER_Y, State.OTHER_MOVIE_DETAIL, '{Nice choice,Good choice, Cool,Wonderful,Wow, Awesome}"! An overview of" $movie"is that" #OTHER_MOVIE_DESC(movie) "Would you like to know another one,like" $movie={#OTHER_MOVIE(movie)} "?"')
-df.add_system_transition(State.OTHER_MOVIE_ANOTHER_N, State.END, '')
+df.add_system_transition(State.OTHER_MOVIE_ANOTHER_N, State.PETS_PROS, '{Ok, Alright, Then, Well}", I know lots of people have pets. What do you think are the benefits of keeping a pet?"')
+
+####################### Pet Pros #######################################################################################################################################################
+# User Turn
+df.add_user_transition(State.PETS_PROS, State.PETS_PROS_DB, '[$pros=#CATCH_PROS()]')
+df.add_user_transition(State.PETS_PROS, State.PETS_PROS_N, '[#CATCH_NO()]')
+df.set_error_successor(State.PETS_PROS, State.PETS_PROS_UNKNOWN)
+
+# System Turn
+df.add_system_transition(State.PETS_PROS_DB, State.END, '"Yes, I think so."#CATCH_PROS_DESC(pros)')
+df.add_system_transition(State.PETS_PROS_N, State.END, '"I understand. Although they may bring allergies or schedule disorder, we can\'t deny that they also bring so much joy into our lives."')
+df.add_system_transition(State.PETS_PROS_UNKNOWN, State.END, '"Interesting idea. I have to say that with almost no effort at all, pets manage to bring so much joy into our lives. They make us laugh, comfort us when we’re sick or upset, and are always there for us no matter what."')
 
 ####################### End Pet Component ##############################################################################################################################################
+# END
+# df.set_error_successor(State.PETS_PROS_ANS, State.PETS_END)
+# df.add_system_transition(State.PETS_END, State.END, '"I\'m glad to talk with you. What other topics would you like to talk about?"')
 
 df.update_state_settings(State.END, system_multi_hop=True)
 # df.add_system_transition(State.END, State.START, '" "')
